package controller;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.geom.RectangularShape;

/**
 * De quoi faire une pré-selection pour ensuite créer une figure dans un dessin
 * dans les classes dérivées par exemple...
 * 
 * @author jc
 * 
 */
public class EcouteurSelection extends EcouteurSouris {

	private Point startPoint;
	private RectangularShape clip;
	private boolean dragging;
	private Color borderColor;
	private Color backgroundColor;

	public EcouteurSelection(MainController mainController) {
		super(mainController);
		mainController.getZoneDeDessin().seteSelection(this);
		this.clip = new Rectangle();
		borderColor = new Color(79, 129, 189);
		backgroundColor = new Color(205, 220, 237, 100);
	}

	public EcouteurSelection(MainController mainController, RectangularShape clip) {
		super(mainController);
		this.clip = clip;
	}

	public RectangularShape getClip() {
		return clip;
	}

	public void setClip(RectangularShape clip) {
		this.clip = clip;
	}

	@Override
	public void mousePressed(MouseEvent e) {
		if (startPoint != null) {
			setClipFrame(startPoint, startPoint);
		}
		startPoint = e.getPoint();
		mainController.getZoneDeDessin().setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
		dragging = true;
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		dragging = false;
		mainController.getZoneDeDessin().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		if (dragging)
			setClipFrame(startPoint, e.getPoint());
	}

	public void drawClip(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		if (clip != null) {
			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			g2d.setPaint(backgroundColor);
			g2d.fill(clip);
			g2d.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 10, new float[] { 4, 2 }, 0));
			g2d.setPaint(borderColor);
			g2d.draw(clip);
		}
	}

	public void setClipFrame(final Point start, final Point end) {
		clip.setFrameFromDiagonal(start, end);
		mainController.getZoneDeDessin().repaint();
	}

	public boolean isValid() {
		return dragging;
	}

	public Color getBorderColor() {
		return borderColor;
	}

	public void setBorderColor(Color borderColor) {
		this.borderColor = borderColor;
	}

	public Color getBackgroundColor() {
		return backgroundColor;
	}

	public void setBackgroundColor(Color backgroundColor) {
		this.backgroundColor = backgroundColor;
	}
}
