package controller;

import static controller.AppProperties.ABOUT;
import static controller.AppProperties.CLEAR;
import static controller.AppProperties.ELLIPTIC_SELECTION;
import static controller.AppProperties.EXIT;
import static controller.AppProperties.OPEN;
import static controller.AppProperties.PEN_SIZE;
import static controller.AppProperties.PEN_TOOL;
import static controller.AppProperties.RECTANGLE_SELECTION;
import static controller.AppProperties.SAVE;
import static controller.AppProperties.SAVE_AS;
import static controller.AppProperties.STAR_TOOL;

import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowEvent;
import java.awt.geom.Ellipse2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import model.Dessin;
import technics.DessinDAOStream;
import technics.IDessinDAO;
import view.AboutDialog;
import view.PenSizeAndColorDialog;
import view.FenetreDeDessin;
import view.ZoneDeDessin;

public class MainController implements KeyListener, MouseMotionListener,
		PropertyChangeListener {
	private FenetreDeDessin fenetreDeDessin;
	private ZoneDeDessin zoneDeDessin;
	private EcouteurCrayon crayon;
	private EcouteurEtoile etoile;
	private EcouteurSouris courant;
	private EcouteurSelection eSelection;
	private Dessin dessin;
	private boolean dirty;
	private int thick;
	private IDessinDAO dessinDao;

	public MainController(Dessin dessin, FenetreDeDessin window) {
		this.fenetreDeDessin = window;
		this.zoneDeDessin = fenetreDeDessin.getDrawPanel();
		this.dessin = dessin;
		zoneDeDessin.setDessin(dessin);
		initOutils();
		courant = null;
		changerOutil(crayon);
		addListeners();
		fenetreDeDessin.setVisible(true);
	}

	private void initOutils() {
		crayon = new EcouteurCrayon(this);
		etoile = new EcouteurEtoile(this);
		eSelection = new EcouteurSelection(this);
	}

	private void addListeners() {
		fenetreDeDessin.addKeyListener(this);
		zoneDeDessin.addMouseMotionListener(this);
		fenetreDeDessin.addPropertyChangeListener(this);
		fenetreDeDessin.addWindowListener(new MainWindowListener(this));
	}

	public void changerOutil(EcouteurSouris outil) {
		zoneDeDessin.removeMouseListener(courant);
		zoneDeDessin.removeMouseMotionListener(courant);
		zoneDeDessin.addMouseListener(outil);
		zoneDeDessin.addMouseMotionListener(outil);
		courant = outil;
		fenetreDeDessin.getSb().afficheOutil(outil.getClass().getSimpleName());
	}

	/* en tant que KeyListener */
	public void keyPressed(KeyEvent e) {
	}

	public void keyReleased(KeyEvent e) {
	}

	public void keyTyped(KeyEvent e) {
		switch (e.getKeyChar()) {
		case 'e':
			changerOutil(etoile);
			break;
		case 'c':
			changerOutil(crayon);
			break;
		case 'd':
			changerOutil(eSelection);
			break;
		}
	}

	@Override
	public void mouseDragged(MouseEvent evt) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseMoved(MouseEvent evt) {
		fenetreDeDessin.getSb().afficheCoordonnees(evt.getX(), evt.getY());

	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		switch (evt.getPropertyName()) {

		case ABOUT:
			AboutDialog.openDialog(true);
			break;

		case EXIT:
			fenetreDeDessin.dispatchEvent(new WindowEvent(fenetreDeDessin, WindowEvent.WINDOW_CLOSING));
			break;

		case CLEAR:
			dessin.effacer();
			dirty = true;
			zoneDeDessin.repaint();
			break;

		case PEN_TOOL:
			changerOutil(crayon);
			break;

		case STAR_TOOL:
			changerOutil(etoile);
			break;

		case OPEN:
			open();
			break;

		case SAVE:
			save();
			break;

		case SAVE_AS:
			saveAs();
			break;

		case RECTANGLE_SELECTION:
			eSelection.setClip(new Rectangle());
			changerOutil(eSelection);
			break;

		case ELLIPTIC_SELECTION:
			eSelection.setClip(new Ellipse2D.Float());
			changerOutil(eSelection);
			break;

		case PEN_SIZE:
			if (PenSizeAndColorDialog.openDialog()) {
				setThick(PenSizeAndColorDialog.getValue());
			}
			break;

		default:
			break;
		}
	}

	public void open() {
		if (closeDessinQuery("Sauvegarde avant ouverture")) {
			JFileChooser openFile = new JFileChooser();
			int returnVal = openFile.showOpenDialog(fenetreDeDessin);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				try {
					dessinDao = new DessinDAOStream(openFile.getSelectedFile().getAbsolutePath());
					dessin = dessinDao.read();
					dirty = false;
					fenetreDeDessin.getSb().showMessage(openFile.getSelectedFile().getName());
				} catch (Exception e) {
					fenetreDeDessin.getSb().showErrorMessage("Erreur de lecture du fichier");
				}
			}
			setDessin(dessin);
			zoneDeDessin.repaint();
		}

	}

	public void setDessin(Dessin dessin) {
		this.dessin = dessin;
		zoneDeDessin.setDessin(dessin);
	}

	public Dessin getDessin() {
		return dessin;
	}

	public int getThick() {
		return thick;
	}

	public FenetreDeDessin getFenetreDeDessin() {
		return fenetreDeDessin;
	}

	public ZoneDeDessin getZoneDeDessin() {
		return zoneDeDessin;
	}

	public void setDirty(boolean dirty) {
		this.dirty = dirty;
	}

	public boolean isDirty() {
		return dirty;
	}

	public void setThick(int thick) {
		this.thick = thick;
		zoneDeDessin.setThick(thick);
	}

	public boolean save() {
		boolean result = false;
		if (dessinDao != null) {
			try {
				dessinDao.write(dessin);
				fenetreDeDessin.getSb().showMessage(dessinDao + " enregistré");
				dirty = false;
				result = true;
			} catch (Exception e) {
				fenetreDeDessin.getSb().showErrorMessage("Erreur d'écriture du fichier");
			}
		} else
			result = saveAs();
		return result;
	}

	public boolean saveAs() {
		boolean result = false;
		JFileChooser saveFile = new JFileChooser();
		int returnVal = saveFile.showSaveDialog(fenetreDeDessin);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			try {
				dessinDao = new DessinDAOStream(saveFile.getSelectedFile().getAbsolutePath());
				dessinDao.write(dessin);
				dirty = false;
				fenetreDeDessin.getSb().showMessage(dessinDao + " enregistré");
				result = true;
			} catch (Exception e) {
				fenetreDeDessin.getSb().showErrorMessage("Erreur d'écriture du fichier");
			}
		}
		return result;
	}

	public boolean closeDessinQuery(String actionName) {
		boolean result = true;
		if (dirty) {
			switch (JOptionPane.showConfirmDialog(this.fenetreDeDessin, "L'opération fait perdre les modifications non enregistrées !\nVoulez-vous enregistrer le dessin ?", actionName, JOptionPane.YES_NO_CANCEL_OPTION)) {
			case JOptionPane.YES_OPTION:
				result = save();
			case JOptionPane.NO_OPTION:
				break;
			default:
				result = false;
			}
		}
		return result;
	}
}
