package view;

import java.awt.BorderLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;

import model.Dessin;
import controller.AppProperties;

/**
 * @author jcheron
 * 
 */
public class FenetreDeDessin extends JFrame {

	private static final long serialVersionUID = 1L;
	private StatusBar sb;
	private ZoneDeDessin drawPanel;
	private JRadioButtonMenuItem mnItemCrayon;
	private JRadioButtonMenuItem mnItemEtoile;
	private JMenuItem menuItemQuitter;

	public FenetreDeDessin() throws HeadlessException {
		super();
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		setLayout(new BorderLayout());
		createDrawPanel();
		createStatusBar();
		createMenu();
		setSize(300, 400);
	}

	/**
	 * Création du menu
	 */
	private void createMenu() {
		JMenuBar menuBar = new JMenuBar();
		JMenu menu = new JMenu("Dessin");
		menuBar.add(menu);
		JMenuItem menuItem = addMenuItem(menu, "Ouvrir...", AppProperties.OPEN);

		menu.addSeparator();
		menuItem = addMenuItem(menu, "Enregistrer", AppProperties.SAVE);
		menuItem = addMenuItem(menu, "Enregistrer sous...", AppProperties.SAVE_AS);
		menu.addSeparator();
		addMenuItem(menu, "Effacer", AppProperties.CLEAR);
		menu.addSeparator();
		addMenuItem(menu, "Quitter", AppProperties.EXIT);
		setJMenuBar(menuBar);
		menu = new JMenu("Outil");
		menuBar.add(menu);
		ButtonGroup btnGroup = new ButtonGroup();
		mnItemCrayon = new JRadioButtonMenuItem("Crayon");
		btnGroup.add(mnItemCrayon);
		addMenuItem(menu, mnItemCrayon, AppProperties.PEN_TOOL);

		mnItemEtoile = new JRadioButtonMenuItem("Etoile");
		btnGroup.add(mnItemEtoile);
		addMenuItem(menu, mnItemEtoile, AppProperties.STAR_TOOL);
		menuItem = new JMenu("Selection");
		menu.add(menuItem);
		btnGroup.add(addMenuItem(menuItem, new JRadioButtonMenuItem("Rectangle"), AppProperties.RECTANGLE_SELECTION));
		btnGroup.add(addMenuItem(menuItem, new JRadioButtonMenuItem("Ellipse"), AppProperties.ELLIPTIC_SELECTION));

		btnGroup.setSelected(mnItemCrayon.getModel(), true);
		addMenuItem(menu, "Epaisseur", AppProperties.PEN_SIZE);
		menu = new JMenu("?");
		menuBar.add(menu);
		menuItem = addMenuItem(menu, "A propos de...", AppProperties.ABOUT);
	}

	private JMenuItem addMenuItem(JMenuItem menu, JMenuItem mnItem, final String action) {
		menu.add(mnItem);
		mnItem.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				firePropertyChange(action, null, null);
			}
		});
		return mnItem;
	}

	private JMenuItem addMenuItem(JMenuItem menu, String caption, final String action) {
		return addMenuItem(menu, new JMenuItem(caption), action);
	}

	private JMenuItem addMenuItem(JMenuItem menu, String caption) {
		return (JMenuItem) menu.add(new JMenuItem(caption));
	}

	/**
	 * Création de la barre d'état
	 */
	private void createStatusBar() {
		sb = new StatusBar();
		add(sb, BorderLayout.SOUTH);
	}

	/**
	 * Création de la zone de dessin ( classe Héritant de Jpanel)
	 */
	private void createDrawPanel() {
		drawPanel = new ZoneDeDessin();
		add(drawPanel, BorderLayout.CENTER);
	}

	public void setDessin(Dessin dessin) {
		drawPanel.setDessin(dessin);
	}

	public StatusBar getSb() {
		return sb;
	}

	public ZoneDeDessin getDrawPanel() {
		return drawPanel;
	}

	public JRadioButtonMenuItem getMnItemCrayon() {
		return mnItemCrayon;
	}

	public JRadioButtonMenuItem getMnItemEtoile() {
		return mnItemEtoile;
	}

	public JMenuItem getMenuItemQuitter() {
		return menuItemQuitter;
	}
}
