package view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;

import javax.swing.JPanel;

import model.Dessin;
import model.Etoile;
import model.Figure;
import model.Trace;
import controller.EcouteurSelection;

/**
 * Correspond à une zone de dessin (drawPanel)
 * 
 * @author heronj
 * 
 */
public class ZoneDeDessin extends JPanel {

	private static final long serialVersionUID = 1L;
	private Dessin dessin;
	private Stroke stroke;
	private EcouteurSelection eSelection;

	public ZoneDeDessin() {
		super();
		setDoubleBuffered(true);
		stroke = new BasicStroke(1);
	}

	public void setDessin(Dessin dessin) {
		this.dessin = dessin;
	}

	public void dessineTrait(int x1, int y1, int x2, int y2) {
		Graphics2D g = (Graphics2D) getGraphics();
		g.setStroke(stroke);
		g.drawLine(x1, y1, x2, y2);
	}

	public void dessineFigure(Figure figure, Graphics g) {
	}

	public void dessineFigure(Trace trace, Graphics g) {
		Point pt = trace.element(0);
		Graphics2D g2d = (Graphics2D) g;
		g2d.setStroke(new BasicStroke(trace.getThick()));
		int xd, yd, x, y;
		xd = (int) (pt.getX());
		yd = (int) (pt.getY());
		for (int numPt = 1; numPt < trace.nbPoints(); numPt++) {
			pt = trace.element(numPt);
			x = (int) (pt.getX());
			y = (int) (pt.getY());
			g.drawLine(xd, yd, x, y);
			xd = x;
			yd = y;
		}
	}

	public void dessineFigure(Etoile etoile, Graphics g) {
		Point pt = etoile.centre();
		Graphics2D g2d = (Graphics2D) g;
		g2d.setStroke(new BasicStroke(etoile.getThick()));
		int xc, yc, x, y;
		xc = (int) (pt.getX());
		yc = (int) (pt.getY());
		for (int numPt = 0; numPt < etoile.nbPoints(); numPt++) {
			pt = etoile.element(numPt);
			x = (int) (pt.getX());
			y = (int) (pt.getY());
			g.drawLine(xc, yc, x, y);
		}
	}

	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		super.paintComponent(g2d);
		setBackground(Color.white);
		g2d.clearRect(0, 0, 600, 600);
		Figure f;
		for (int numFig = 0; numFig < dessin.nbFigures(); numFig++) {
			f = dessin.element(numFig);
			if (f instanceof Trace)
				dessineFigure((Trace) f, g);
			if (f instanceof Etoile)
				dessineFigure((Etoile) f, g);
		}
		if (eSelection != null && eSelection.isValid()) {
			eSelection.drawClip(g2d);
		}
	}

	public void clear() {
		dessin.effacer();
		repaint();
	}

	public void seteSelection(EcouteurSelection eSelection) {
		this.eSelection = eSelection;
	}

	public void setThick(int value) {
		stroke = new BasicStroke(value);
	}
}
